#!/usr/bin/env python

"""
Synopsis:
    Convert hdf5 file to XML or the reverse.
Usage:
    ./obj_hdf_xml.py <command> infile outfile
Commands:
    hdf2xml
    xml2hdf
Examples:
    ./obj_hdf_xml.py xml2hdf infile.xml outfile.hdf5
    ./obj_hdf_xml.py -f xml2hdf infile.xml outfile.hdf5
    ./obj_hdf_xml.py hdf2xml infile.hdf5 outfile.xml
Notes:
    Run `./obj_hdf_xml --help` for help with options etc.
"""

import sys
import os
import argparse
from lxml import objectify
import h5py
import numpy
from subprocess import Popen, PIPE


def hdf2xml(infilename, outfilename, options):
    """Dump an HDF5 file to XML.  Use `h5dump -x` to create XML output.
    """
    args = ['h5dump', '-x', infilename]
    proc = Popen(args, stdout=PIPE)
    output = proc.stdout.read()
    with open(outfilename, 'w') as outfile:
        outfile.write(output)


def xml2hdf(infilename, outfilename, options):
    """Parse an XML file (created by `h5dump -x`) and create an HDF5 file.
    Use lxml.objectify to parse and access data in the XML file.
    Use h5py to create the HDF5 file.
    """
    doc = objectify.parse(infilename)
    root = doc.getroot()
    outfile = h5py.File(outfilename, "w")
    rootgroup = root.RootGroup
    xml2hdf_objects(rootgroup, outfile, options)


def xml2hdf_objects(group, ogroup, options):
    """Process the dataset and group object under a group.
    If a child object is a group, process its children, too, recursively.
    """
    for element in group.iterchildren():
        if element.tag.endswith('Dataset'):
            xml2hdf_dataset(element, ogroup, options)
        elif element.tag.endswith('Group'):
            ogroup1 = xml2hdf_group(element, ogroup, options)
            xml2hdf_objects(element, ogroup1, options)


def xml2hdf_group(group, object, options):
    """Process and create a single group."""
    name = group.get('Name')
    ogroup = object.create_group(name)
    return ogroup


def xml2hdf_dataset(dataset, ogroup, options):
    """Process and create a dataset."""
    datasetname = dataset.get('Name')
    datatype = dataset.DataType
    atomictype = datatype.AtomicType
    if hasattr(atomictype, 'IntegerType'):
        dtype = numpy.int
        #size = itype.get('Size')
        #sign = itype.get('Sign')
    elif hasattr(atomictype, 'FloatType'):
        dtype = numpy.float
        #size = itype.get('Size')
        #sign = itype.get('Sign')
    datafromfile = dataset.Data.DataFromFile
    # use .text to get python str (not objectify.StringElement)
    # from DataFromFile.
    datastr = datafromfile.text
    ditems = datastr.splitlines()
    if dtype is numpy.int:
        ditems = [int(ditem) for ditem in ditems if ditem.strip()]
    elif dtype is numpy.float:
        ditems = [float(ditem) for ditem in ditems if ditem.strip()]
    else:
        sys.exit('error: dataset contains data of unknown type')
    odataset = ogroup.create_dataset(datasetname, (len(ditems),), dtype=dtype)
    nparray = numpy.array(ditems, dtype=dtype)
    odataset[:] = nparray


def main():
    parser = argparse.ArgumentParser(
        description=__doc__,
        formatter_class=argparse.RawDescriptionHelpFormatter,)
    parser.add_argument(
        'command', type=str,
        choices=['xml2hdf', 'hdf2xml', ],
        help='command')
    parser.add_argument(
        'infile', type=str,
        help='name of input file to read from')
    parser.add_argument(
        'outfile', type=str,
        help='name of output file to write to')
    parser.add_argument(
        '-f', '--force', action='store_true', default=False,
        help='force overwrite of output file')
    options = parser.parse_args()
    if not options.force and os.path.exists(options.outfile):
        parser.error('out file exists; use -f to overwrite')
    if options.command == 'xml2hdf':
        xml2hdf(options.infile, options.outfile, options)
    elif options.command == 'hdf2xml':
        hdf2xml(options.infile, options.outfile, options)


if __name__ == '__main__':
    #import ipdb; ipdb.set_trace()
    main()
